# Perl -> Caliper.pl
# This program reads and converts to human readable the value of a Cen-Tech scale via my adapter.
# http://www.wire2wire.org/milling/cen-tech.html
# all rights reserved Peter Shabino. 

# PJS 24MAR09 New

# configuration vars
$COMPORT = "COM1";
$MMFACTOR = .01;
$INCHFACTOR = .0005; 


######################################################################
# init serial stuff
print("Init serial port \n");
$COMPORT = SETUP_SERIAL($COMPORT);

print("	Flush \n");
#flush serial port buffers
$sendstring = "         ";		 		# garbage chars at start are to just flush out port
$count_out = $COMPORT->write($sendstring); 		# actual send of data
if(length($sendstring) != $count_out){ 			# error check send fail if bad
	print("Error send failed!!\n",$count_out," Sent out of ",length($sendstring),"\n");
	exit(-1);
}

print("	Purge \n");
# this code clears the read buffer (just read all the chars in until its empty)
$recv_length = 1; 	# read one at a time
$count_in = 1;		# inital value to start while loop
while ($count_in == 1){						
	($count_in, $recvstring) = $COMPORT->read($recv_length);	# read it
}

######################################################################
# Loop till E is pressed
while($input !~ m/e/i){
	print("get reading \n");
	$sendstring = "G"; 	
	$count_out = $COMPORT->write($sendstring); 		# actual send of data
	if(length($sendstring) != $count_out){ 			# error check send fail if bad
		print("TX error on serial port");
		exit(-1);
	}
	$recv_length = 6;						# how many chars to get
	($count_in, $recvstring) = $COMPORT->read($recv_length);	#actual read
	if($count_in != $recv_length || $recvstring !~ m/^G/ ){	
		print("RX error on serial port\n");
		print("count: ",$count_in,"\n");
		print("data: \n",unpack("H*",$recvstring),"\n");
		exit(-1);
	}
	#print("raw data: \n",unpack("H*",substr($recvstring, 1, 3)),"\n");
	$status = substr(unpack("B*",substr($recvstring, 1, 3)), 20);
	#print("status: \n",$status,"\n");
	$temp = substr(unpack("B*",substr($recvstring, 1, 3)),0 ,20);
	@bits = split(//,$temp);
	@bits = reverse(@bits);
	#print("data: \n",join(//,@bits),"\n");
	$prescale = unpack("H*",pack("B*","0000".join(//,@bits)));
	$prescale = hex($prescale);
	#print("prescale data: \n",$prescale,"\n");
	$units = "";
	if(substr($status,3,1) == 0){
		$reading = $prescale * $MMFACTOR; # mm conversion
		$units = "mm";
	}else{
		$reading = $prescale * $INCHFACTOR; # inch conversion
		$units = "\"";
	}
	if(substr($status,0,1) == 1){
		$reading = $reading * -1; # add negitive sign if needed
	}
	
	print("	Scale = ".$reading.$units."\n");
	print("type \"E\" and enter to quit or just enter to take another reading.\n>");
	$input = <STDIN>;
}

print("DONE \n");
exit(0);

###############################################
# this function sets up the serial port
###############################################
sub SETUP_SERIAL{
	my $PortName = $_[0];

	my $COMPORT = "";

	require 5.003;
	use Win32::SerialPort qw( :STAT 0.19 ); 
	#more info at http://search.cpan.org/~bbirth/Win32-SerialPort-0.19/lib/Win32/SerialPort.pm

	# open the serial port	
	$COMPORT = new Win32::SerialPort ($PortName) || die "Can't open $PortName: $^E\n";    

	# configure the port
	$COMPORT->baudrate(19200);
	$COMPORT->parity("none");
	$COMPORT->databits(8);
	$COMPORT->stopbits(1);
	$COMPORT->handshake("");
	$COMPORT->binary(T);          		# just say Yes (Win 3.x option)
#	$COMPORT->parity_enable(1);   
	$COMPORT->debug(0);
	$COMPORT->write_char_time(50);		# needed for write timeouts
	$COMPORT->write_const_time(1000);	# needed for write timeouts
	$COMPORT->read_interval(1000);    	# needed for read timeouts max time between read char (milliseconds)
	$COMPORT->read_char_time(5);     	# needed for read timeouts avg time between read char
	$COMPORT->read_const_time(1000);  	# needed for read timeouts total = (avg * bytes) + const 
	$COMPORT->buffers(4096, 4096);
	$COMPORT->write_settings;			# actualy use the settings above

	return($COMPORT);
}


